/*
maya/projects/arnold_src+c++/dd_sideMask.cpp
*/
#include <ai.h>
#include <cstring>
#include "utilities.h"


AI_SHADER_NODE_EXPORT_METHODS(SampleMethods);
namespace {
    enum paramIndex { k_front_color, k_rear_color, k_swap, k_cross_dissolve};
    };

node_parameters {
    AiParameterRGB("frontColor", 0.7f, 0.7f, 0);
    AiParameterRGB("rearColor", 0.7f, 0, 0);
    AiParameterBool("swap", 0);
    AiParameterFlt("cross_dissolve", 0);
    }

shader_evaluate {
    AtRGB front = AiShaderEvalParamRGB(k_front_color);
    AtRGB rear = AiShaderEvalParamRGB(k_rear_color);
    bool  swap = AiShaderEvalParamBool(k_swap);
    float dissolve = AiShaderEvalParamFlt(k_cross_dissolve);

    // incoming direction of the "camera ray", already
    // normalized, but it must be reversed so that it is
    // tail-to-tail with the surface normal.

    // Geometric surface normal, but we have to normalize
    // the vector otherwise we get the wrong angle (as the
    // cosine) when we use the dot product.
    float dot = AiV3Dot(-(sg->Rd), AiV3Normalize(sg->N));
    AtRGB a, b;

    if(dot < 0) {
        a = (swap) ? front : rear;
        b = (swap) ? rear : front;
        }
    else
        {
        a = (swap) ? rear : front;
        b = (swap) ? front : rear;
        }

    sg->out.RGB() = mix(a, b, dissolve);
    }
node_loader {
    if (i > 0)
        return false; 
    node->methods        = SampleMethods;
    node->output_type    = AI_TYPE_RGB;
	node->name           = "dd_SideMask";
    node->node_type      = AI_NODE_SHADER;
    strcpy(node->version, AI_VERSION);
    return true;
	}
	
// The remaining macros can be left "empty"
node_initialize { }
node_update { }
node_finish { }
